package com.ctg.ag.sdk.core;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.ctg.ag.sdk.core.HttpClient;
import com.ctg.ag.sdk.core.constant.ParamPosition;
import com.ctg.ag.sdk.core.exception.SdkClientException;
import com.ctg.ag.sdk.core.http.HttpClientFactory;
import com.ctg.ag.sdk.core.http.RequestFormat;
import com.ctg.ag.sdk.core.model.ApiCallBack;
import com.ctg.ag.sdk.core.model.BaseApiRequest;
import com.ctg.ag.sdk.core.model.BaseApiResponse;
import com.ctg.ag.sdk.core.model.BuilderParams;
import com.ctg.ag.sdk.core.util.Signature;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;
import java.util.TreeMap;
import java.util.concurrent.Future;

public abstract class BaseApiClient {
  protected final String appKey;
  
  protected final String appSecret;
  
  protected final String regionId;
  
  protected final String appServerId;
  
  protected final boolean isEcho;
  
  protected final HttpClient httpClient;
  
  protected static long offset = 0L;
  
  protected static long lastGetOffsetTime = 0L;
  
  protected final String host;
  
  protected final String scheme;
  
  protected final String server;
  
  public BaseApiClient(BuilderParams builderParams) {
    this.appKey = builderParams.getAppKey();
    this.appSecret = builderParams.getAppSecret();
    this.httpClient = HttpClientFactory.buildClient(builderParams);
    this.host = builderParams.getHost();
    this.scheme = builderParams.getScheme().getValue();
    this.server = builderParams.getServer();
    this.regionId = builderParams.getRegionId();
    this.appServerId = builderParams.getAppServerId();
    this.isEcho = builderParams.isEcho();
    offset = getOffset();
  }
  
  private long getOffset() {
    long current = System.currentTimeMillis();
    if (current - lastGetOffsetTime > 600000L) {
      offset = timeSync(this.server);
      lastGetOffsetTime = current;
    } 
    return offset;
  }
  
  private long timeSync(String server) {
    if (this.isEcho) {
      Object object = new Object(this, RequestFormat.GET(), null, new BaseApiRequest.Meta[0]);
      if (this.regionId != null) {
        object.getGlobalParam().put("X-ROUTE-REGION", this.regionId);
        object.getGlobalParam().put("X-ROUTE-APP-ID", this.appServerId);
      } 
      object.setHost(server);
      object.setPath("/echo");
      try {
        long start = System.currentTimeMillis();
        BaseApiResponse response = this.httpClient.syncInvoke((BaseApiRequest)object);
        long end = System.currentTimeMillis();
        Map<String, String> headers = response.getHeaders();
        long serviceTime = Double.valueOf(headers.get("x-ag-timestamp")).longValue();
        return serviceTime - (end + start) / 2L;
      } catch (Exception e) {
        throw new RuntimeException("Unable to synchronize time from server: " + object, e);
      } 
    } 
    return offset;
  }
  
  private <REQ extends BaseApiRequest> REQ buildSDKRequest(REQ apiReq) throws Exception {
    String pathWithPathParameter = combinePathParam(apiReq.getPath(), apiReq.getParam(new ParamPosition[] { ParamPosition.PATH }));
    apiReq.setPath(pathWithPathParameter);
    Date current = new Date(System.currentTimeMillis() + getOffset());
    apiReq.getGlobalParam().put("timestamp", String.valueOf(current.getTime()));
    apiReq.getGlobalParam().put("application", this.appKey);
    if (this.appSecret != null && !this.appSecret.isEmpty()) {
      Map<String, List<String>> input = new TreeMap<>();
      for (String s : apiReq.getMeta().keySet()) {
        List<String> value = apiReq.getParams(s);
        input.put(s, value);
      } 
      apiReq.getGlobalParam().put("signature", Signature.sign(this.appSecret, (String)apiReq.getGlobalParam().get("application"), (String)apiReq
            .getGlobalParam().get("timestamp"), input, apiReq.getBody()));
    } 
    apiReq.getGlobalParam().put("Date", getHttpDateHeaderValue(current));
    apiReq.getGlobalParam().put("User-Agent", "Telecom API Gateway Java SDK");
    apiReq.getGlobalParam().put("version", apiReq.getGlobalParam().get("version"));
    apiReq.getGlobalParam().put("sdk", "GIT: a4fb7fca");
    apiReq.getGlobalParam().put("Content-Type", apiReq.getMethod().getContentType().toString());
    if (apiReq.getGlobalParam().get("version") == null || ((String)apiReq.getGlobalParam().get("version")).trim().isEmpty())
      apiReq.getGlobalParam().remove("version"); 
    return apiReq;
  }
  
  private String combinePathParam(String path, Map<String, List<String>> pathParams) {
    if (pathParams == null)
      return path; 
    for (String key : pathParams.keySet())
      path = path.replace("[" + key + "]", ((List<CharSequence>)pathParams.get(key)).iterator().next()); 
    return path;
  }
  
  private String getHttpDateHeaderValue(Date date) {
    SimpleDateFormat dateFormat = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z", Locale.US);
    dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
    return dateFormat.format(date);
  }
  
  public final <R extends BaseApiResponse> R syncInvoke(BaseApiRequest apiRequest) throws Exception {
    try {
      apiRequest = buildSDKRequest(apiRequest);
      return (R)this.httpClient.syncInvoke(apiRequest);
    } catch (IOException e) {
      throw new SdkClientException(e);
    } 
  }
  
  public final <REQ extends BaseApiRequest, RESP extends BaseApiResponse> Future<RESP> asyncInvoke(REQ apiRequest, ApiCallBack<REQ, RESP> callback) {
    try {
      apiRequest = buildSDKRequest(apiRequest);
      return this.httpClient.asyncInvoke((BaseApiRequest)apiRequest, callback);
    } catch (Exception e) {
      throw new SdkClientException(e);
    } 
  }
  
  public void shutdown() {
    try {
      if (this.httpClient != null)
        this.httpClient.close(); 
    } catch (Exception exception) {}
  }
}
